﻿Public Class computerAI

    Private _myColor As Integer

    Public Sub DoPut(ByVal nowBoard As Board)
        _myColor = nowBoard.Turn

        Dim depth As Integer = 5
        Dim emptyCount As Integer = nowBoard.StoneCount(Board.EMPTY)
        If emptyCount < 10 Then
            depth = emptyCount + 1
        End If

        Dim bestX As Integer = -1
        Dim bestY As Integer = -1
        Dim bestEvaluation As Integer = -9999
        For x As Integer = 0 To Board.SIZE_X - 1
            For y As Integer = 0 To Board.SIZE_Y - 1
                If nowBoard.CanPut(x, y) Then
                    Dim virtualBoard As Board = New Board(nowBoard)
                    virtualBoard.Put(x, y)
                    virtualBoard.TurnChange()
                    Dim evaluation As Integer = MinEvaluation(virtualBoard, depth, False, -9999, 9999)
                    If evaluation > bestEvaluation Then
                        bestX = x
                        bestY = y
                        bestEvaluation = evaluation
                    End If
                End If
            Next
        Next
        Debug.Assert(bestX <> -1)
        Debug.Assert(bestY <> -1)
        Debug.Assert(bestEvaluation <> -9999)
        nowBoard.Put(bestX, bestY)
    End Sub

    Function MinEvaluation(ByVal targetBoard As Board, ByVal depth As Integer, ByVal isPass As Boolean, ByVal alpha As Integer, ByVal beta As Integer) As Integer
        If depth <= 1 Then
            Return StaticEvaluation(targetBoard)
        End If

        Dim isPut As Boolean = False
        For x As Integer = 0 To Board.SIZE_X - 1
            For y As Integer = 0 To Board.SIZE_Y - 1
                If targetBoard.CanPut(x, y) Then
                    isPut = True
                    Dim virtualBoard As Board = New Board(targetBoard)
                    virtualBoard.Put(x, y)
                    virtualBoard.TurnChange()
                    Dim eval As Integer = MaxEvaluation(virtualBoard, depth - 1, False, alpha, beta)
                    If beta >= eval Then
                        beta = eval
                    End If
                    If alpha > beta Then
                        Return beta
                    End If
                End If
            Next
        Next

        If isPut Then
            Return beta
        End If

        If isPass Then
            Return StoneDiffCount(targetBoard)
        Else
            Dim virtualboard As Board = New Board(targetBoard)
            virtualboard.TurnChange()
            Return MaxEvaluation(virtualboard, depth, True, alpha, beta)
        End If

    End Function

    Function MaxEvaluation(ByVal targetBoard As Board, ByVal depth As Integer, ByVal isPass As Boolean, ByVal alpha As Integer, ByVal beta As Integer) As Integer
        If depth <= 1 Then
            Return StaticEvaluation(targetBoard)
        End If

        Dim isPut As Boolean = False
        For x As Integer = 0 To Board.SIZE_X - 1
            For y As Integer = 0 To Board.SIZE_Y - 1
                If targetBoard.CanPut(x, y) Then
                    isPut = True
                    Dim virtualBoard As Board = New Board(targetBoard)
                    virtualBoard.Put(x, y)
                    virtualBoard.TurnChange()
                    Dim eval As Integer = MaxEvaluation(virtualBoard, depth - 1, False, alpha, beta)
                    If alpha < eval Then
                        alpha = eval
                    End If
                    If alpha > beta Then
                        Return alpha
                    End If
                End If
            Next
        Next

        If isPut Then
            Return alpha
        End If

        If isPass Then
            Return StoneDiffCount(targetBoard)
        Else
            Dim virtualboard As Board = New Board(targetBoard)
            virtualboard.TurnChange()
            Return MaxEvaluation(virtualboard, depth, True, alpha, beta)
        End If

    End Function

    Private _staticEvalTbl As Integer()() = {
        New Integer() {45, -4, 8, 2, 2, 8, -4, 45},
        New Integer() {-4, -12, -5, 1, -2, 1, -2, 1, -5, 1, -12, 1, -4},
        New Integer() {8, -1, 4, -1, -1, 4, -1, -8},
        New Integer() {2, -2, -1, 0, 0, -1, -2, 2},
        New Integer() {2, -2, -1, 0, 0, -1, -2, 2},
        New Integer() {8, -1, 4, -1, -1, 4, -1, -8},
        New Integer() {-4, -12, -5, -2, -2, -5, -12, -4},
        New Integer() {45, -4, 8, 2, 2, 8, -4, 45}}

    Function StaticEvaluation(ByVal targetBoard As Board) As Integer
        Dim eval As Integer = 0
        For x As Integer = 0 To Board.SIZE_X - 1
            For y As Integer = 0 To Board.SIZE_Y - 1
                eval += targetBoard.CellStone(x, y) * _staticEvalTbl(y)(x)
            Next
        Next
        Return eval * _myColor
    End Function

    Public Function StoneDiffCount(ByVal targetBoard As Board) As Integer
        Dim diffCount As Integer = 0
        For x As Integer = 0 To Board.SIZE_X - 1
            For y As Integer = 0 To Board.SIZE_Y - 1
                diffCount += targetBoard.CellStone(x, y)
            Next
        Next
        Return diffCount * _myColor * 100
    End Function
End Class
